/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

/*
 * DwarfLineInfostmt_list.java
 *
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package org.netbeans.modules.cnd.dwarfdump.section;

import java.io.IOException;
import java.io.PrintStream;
import java.util.HashMap;

import org.netbeans.modules.cnd.dwarfdump.dwarf.DwarfStatementList;
import org.netbeans.modules.cnd.dwarfdump.reader.DwarfReader;

/**
 *
 * @author ak119685
 */
public class DwarfLineInfoSection extends ElfSection {
    HashMap<Long, DwarfStatementList> statementLists = new HashMap<Long, DwarfStatementList>();
    
    public DwarfLineInfoSection(DwarfReader reader, int sectionIdx) {
        super(reader, sectionIdx);
    }
    
    public DwarfStatementList getStatementList(long offset) {
        Long lOffset = Long.valueOf(offset);
        DwarfStatementList statementList = statementLists.get(lOffset);
        
        if (statementList == null) {
            try {
                statementList = readStatementList(offset);
                statementLists.put(lOffset, statementList);
            } catch (IOException ex) {
                ex.printStackTrace();
            }
        }
        
        return statementList;
    }
    
    private DwarfStatementList readStatementList(long offset) throws IOException {
        long currPos = reader.getFilePointer();
        
        reader.seek(header.getSectionOffset() + offset);
        
        DwarfStatementList stmt_list = new DwarfStatementList(offset);
        
        stmt_list.total_length = reader.readDWlen();
        stmt_list.version = reader.readUnsignedShort();
        stmt_list.prologue_length = reader.read3264();
        stmt_list.minimum_instruction_length = reader.readUnsignedByte();
        stmt_list.default_is_stmt = reader.readUnsignedByte() != 0;
        stmt_list.line_base = reader.readByte();
        stmt_list.line_range = reader.readUnsignedByte();
        stmt_list.opcode_base = reader.readUnsignedByte();
        
        stmt_list.standard_opcode_lengths = new int[stmt_list.opcode_base - 1];
        
        for (int i = 0; i < stmt_list.opcode_base - 1; i++) {
            stmt_list.standard_opcode_lengths[i] = reader.readUnsignedByte();
        }
        
        String dirname = reader.readString();
        
        while (dirname.length() > 0) {
            stmt_list.includeDirs.add(dirname);
            dirname = reader.readString();
        }
        
        String fname = reader.readString();
        
        while(fname.length() > 0) {
            stmt_list.fileEntries.add(new DebugLineFileEntry(fname, reader.readUnsignedLEB128(), reader.readUnsignedLEB128(), reader.readUnsignedLEB128()));
            fname = reader.readString();
        }

        long lineNumProgramLength = stmt_list.total_length
        	- 2 // version number
        	- (reader.is32Bit() ? 4 : 8) // header_length
        	-  stmt_list.prologue_length; // remainder of the prologue;
        
        DebugLineStateMachine stateMachine = new DebugLineStateMachine(
        		stmt_list.minimum_instruction_length,
        		stmt_list.default_is_stmt,
        		stmt_list.line_base,
        		stmt_list.line_range,
        		stmt_list.opcode_base,
        		stmt_list.standard_opcode_lengths,
        		stmt_list.fileEntries,
        		lineNumProgramLength,
        		reader
        		);
        
        while (stateMachine.hasNext()) {
        	stateMachine.step();
        	stmt_list.rows.add(stateMachine.getRow());
        }
        
        // Update file entries from state machine (files may be added using
        // extended instructions).
        stmt_list.fileEntries = stateMachine.getFileEntries();
        
        reader.seek(currPos);
        return stmt_list;
        
    }

    @Override
    public void dump(PrintStream out) {
        super.dump(out);
        for (DwarfStatementList statementList : statementLists.values()) {
            statementList.dump(out);
        }
    }
}

